@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2015 The ANGLE Project Authors. All rights reserved.
+// Copyright 2015 The ANGLE Project Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -23,15 +23,21 @@ class MockRenderbufferImpl : public RenderbufferImpl
   public:
     MockRenderbufferImpl() : RenderbufferImpl(mMockState) {}
     virtual ~MockRenderbufferImpl() { destructor(); }
-    MOCK_METHOD4(setStorage, angle::Result(const gl::Context *, GLenum, size_t, size_t));
-    MOCK_METHOD5(setStorageMultisample,
-                 angle::Result(const gl::Context *, size_t, GLenum, size_t, size_t));
+    MOCK_METHOD4(setStorage, angle::Result(const gl::Context *, GLenum, GLsizei, GLsizei));
+    MOCK_METHOD6(setStorageMultisample,
+                 angle::Result(const gl::Context *,
+                               GLsizei,
+                               GLenum,
+                               GLsizei,
+                               GLsizei,
+                               gl::MultisamplingMode));
     MOCK_METHOD2(setStorageEGLImageTarget, angle::Result(const gl::Context *, egl::Image *));
 
-    MOCK_METHOD4(getAttachmentRenderTarget,
+    MOCK_METHOD5(getAttachmentRenderTarget,
                  angle::Result(const gl::Context *,
                                GLenum,
                                const gl::ImageIndex &,
+                               GLsizei,
                                FramebufferAttachmentRenderTarget **));
 
     MOCK_METHOD0(destructor, void());
