@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013 Google Inc. All rights reserved.
- * Copyright (C) 2013-2022 Apple Inc.  All rights reserved.
+ * Copyright (C) 2013-2023 Apple Inc.  All rights reserved.
  * Copyright (C) 2013 Nokia Corporation and/or its subsidiary(-ies).
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,10 +44,16 @@
 #if PLATFORM(COCOA)
 #include "CoreAudioCaptureSource.h"
 #include "DisplayCaptureSourceCocoa.h"
+#include "MockAudioSharedUnit.h"
 #include "MockRealtimeVideoSourceMac.h"
 #endif
 
+#if PLATFORM(IOS_FAMILY)
+#include "CoreAudioCaptureSourceIOS.h"
+#endif
+
 #if USE(GSTREAMER)
+#include "MockDisplayCaptureSourceGStreamer.h"
 #include "MockRealtimeVideoSourceGStreamer.h"
 #endif
 
@@ -56,96 +62,151 @@ namespace WebCore {
 static inline Vector<MockMediaDevice> defaultDevices()
 {
     return Vector<MockMediaDevice> {
-        MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 1"_s, MockMicrophoneProperties { 44100 } },
-        MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 2"_s, MockMicrophoneProperties { 48000 } },
+        MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 1"_s, { }, MockMicrophoneProperties { 44100 , { } } },
+        MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 2"_s, { },  MockMicrophoneProperties { 48000, { false } } },
+        MockMediaDevice { "239c24b1-3b15-11e3-8224-0800200c9a66"_s, "Mock audio device 3"_s, { },  MockMicrophoneProperties { 96000, { true } } },
 
-        MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 1"_s, MockSpeakerProperties { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, 44100 } },
-        MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 2"_s, MockSpeakerProperties { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, 48000 } },
-        MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 3"_s, MockSpeakerProperties { String { }, 48000 } },
+        MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 1"_s, { },  MockSpeakerProperties { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, 44100 } },
+        MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 2"_s, { },  MockSpeakerProperties { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, 48000 } },
+        MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 3"_s, { },  MockSpeakerProperties { String { }, 48000 } },
 
-        MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 1"_s,
+        MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 1"_s, { },
             MockCameraProperties {
                 30,
-                RealtimeMediaSourceSettings::VideoFacingMode::User, {
-                    { { 2560, 1440 }, { { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } } },
-                    { { 1280, 720 }, { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
-                    { { 640, 480 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
-                    { { 112, 112 },  { { 30, 30}, { 27.5, 27.5}, { 25, 25}, { 22.5, 22.5}, { 20, 20}, { 17.5, 17.5}, { 15, 15}, { 12.5, 12.5}, { 10, 10}, { 7.5, 7.5}, { 5, 5} } },
+                VideoFacingMode::User, {
+                    { { 2560, 1440 }, { { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } }, 1, 1, true },
+                    { { 1280, 720 }, { { 30, 30 }, { 27.5, 27.5 }, { 25, 25 }, { 22.5, 22.5 }, { 20, 20 }, { 17.5, 17.5 }, { 15, 15 }, { 12.5, 12.5 }, { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } }, 1, 1, true },
+                    { { 640, 480 },  { { 30, 30 }, { 27.5, 27.5 }, { 25, 25 }, { 22.5, 22.5 }, { 20, 20 }, { 17.5, 17.5 }, { 15, 15 }, { 12.5, 12.5 }, { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } }, 1, 1, true },
+                    { { 112, 112 },  { { 30, 30 }, { 27.5, 27.5 }, { 25, 25 }, { 22.5, 22.5 }, { 20, 20 }, { 17.5, 17.5 }, { 15, 15 }, { 12.5, 12.5 }, { 10, 10 }, { 7.5, 7.5 }, { 5, 5 } }, 1, 1, true },
                 },
                 Color::black,
+                { }, // whiteBalanceModes
+                false, // supportsTorch
+                false, // background blur enabled
             } },
 
-        MockMediaDevice { "239c24b3-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 2"_s,
+        MockMediaDevice { "239c24b3-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 2"_s, { },
             MockCameraProperties {
                 15,
-                RealtimeMediaSourceSettings::VideoFacingMode::Environment, {
-                    { { 3840, 2160 }, { { 2, 30 } } },
-                    { { 1920, 1080 }, { { 2, 30 } } },
-                    { { 1280, 720 },  { { 3, 120 } } },
-                    { { 960, 540 },   { { 3, 60 } } },
-                    { { 640, 480 },   { { 2, 30 } } },
-                    { { 352, 288 },   { { 2, 30 } } },
-                    { { 320, 240 },   { { 2, 30 } } },
-                    { { 160, 120 },   { { 2, 30 } } },
+                VideoFacingMode::Environment, {
+                    { { 3840, 2160 }, { { 2, 30 } }, 1, 4, false },
+                    { { 1920, 1080 }, { { 2, 30 } }, 1, 4, true },
+                    { { 1280, 720 },  { { 3, 120 } }, 1, 4, false },
+                    { { 960, 540 },   { { 3, 60 } }, 1, 4, false },
+                    { { 640, 480 },   { { 2, 30 } }, 1, 4, false },
+                    { { 352, 288 },   { { 2, 30 } }, 1, 4, false },
+                    { { 320, 240 },   { { 2, 30 } }, 1, 4, false },
+                    { { 160, 120 },   { { 2, 30 } }, 1, 4, false },
                 },
                 Color::darkGray,
+                { MeteringMode::Manual, MeteringMode::SingleShot, MeteringMode::Continuous },
+                true,
+                true, // background blur enabled
             } },
 
-        MockMediaDevice { "SCREEN-1"_s, "Mock screen device 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::lightGray, { 1920, 1080 } } },
-        MockMediaDevice { "SCREEN-2"_s, "Mock screen device 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::yellow, { 3840, 2160 } } },
+        MockMediaDevice { "SCREEN-1"_s, "Mock screen device 1"_s, { }, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::lightGray, { 1920, 1080 } } },
+        MockMediaDevice { "SCREEN-2"_s, "Mock screen device 2"_s, { }, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::yellow, { 3840, 2160 } } },
 
-        MockMediaDevice { "WINDOW-1"_s, "Mock window device 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 241, 181 }, { 640, 480 } } },
-        MockMediaDevice { "WINDOW-2"_s, "Mock window device 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 208, 181 }, { 1280, 600 } } },
+        MockMediaDevice { "WINDOW-1"_s, "Mock window device 1"_s, { }, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 241, 181 }, { 640, 480 } } },
+        MockMediaDevice { "WINDOW-2"_s, "Mock window device 2"_s, { }, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 208, 181 }, { 1280, 600 } } },
     };
 }
 
 class MockRealtimeVideoSourceFactory : public VideoCaptureFactory {
 public:
-    CaptureSourceOrError createVideoCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
+    CaptureSourceOrError createVideoCaptureSource(const CaptureDevice& device, MediaDeviceHashSalts&& hashSalts, const MediaConstraints* constraints, std::optional<PageIdentifier> pageIdentifier) final
     {
         ASSERT(device.type() == CaptureDevice::DeviceType::Camera);
         if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType::Camera, device.persistentId()))
-            return { "Unable to find mock camera device with given persistentID"_s };
+            return CaptureSourceOrError({ "Unable to find mock camera device with given persistentID"_s, MediaAccessDenialReason::PermissionDenied });
 
-        return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
+        auto mock = MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(device.persistentId());
+        ASSERT(mock);
+        if (mock->flags.contains(MockMediaDevice::Flag::Invalid))
+            return CaptureSourceOrError({ "Invalid mock camera device"_s, MediaAccessDenialReason::PermissionDenied });
+
+        return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalts), constraints, pageIdentifier);
     }
 
 private:
     CaptureDeviceManager& videoCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().videoCaptureDeviceManager(); }
 };
 
-#if PLATFORM(MAC)
-class MockDisplayCapturer final : public DisplayCaptureSourceCocoa::Capturer {
+#if PLATFORM(COCOA)
+class MockDisplayCapturer final
+    : public DisplayCaptureSourceCocoa::Capturer
+    , public CanMakeWeakPtr<MockDisplayCapturer> {
 public:
-    MockDisplayCapturer(const CaptureDevice&, PageIdentifier);
+    MockDisplayCapturer(CapturerObserver&, const CaptureDevice&, std::optional<PageIdentifier>);
+
+    void triggerMockCaptureConfigurationChange();
 
 private:
     bool start() final;
-    void stop() final  { m_source->stop(); }
+    void stop() final;
     DisplayCaptureSourceCocoa::DisplayFrameType generateFrame() final;
-    RealtimeMediaSourceSettings::DisplaySurfaceType surfaceType() const final { return RealtimeMediaSourceSettings::DisplaySurfaceType::Monitor; }
+    DisplaySurfaceType surfaceType() const final { return DisplaySurfaceType::Monitor; }
     void commitConfiguration(const RealtimeMediaSourceSettings&) final;
     CaptureDevice::DeviceType deviceType() const final { return CaptureDevice::DeviceType::Screen; }
     IntSize intrinsicSize() const final;
 #if !RELEASE_LOG_DISABLED
-    const char* logClassName() const final { return "MockDisplayCapturer"; }
+    ASCIILiteral logClassName() const final { return "MockDisplayCapturer"_s; }
 #endif
+    void whenReady(CompletionHandler<void(CaptureSourceError&&)>&&) final;
+
+    void readyTimerFired();
+
     Ref<MockRealtimeVideoSource> m_source;
     RealtimeMediaSourceSettings m_settings;
+    Timer m_readyTimer;
+    bool m_isRunning { false };
+    bool m_didReceiveVideoFrame { false };
+    CompletionHandler<void(CaptureSourceError&&)> m_whenReadyCallback;
 };
 
-MockDisplayCapturer::MockDisplayCapturer(const CaptureDevice& device, PageIdentifier pageIdentifier)
-    : m_source(MockRealtimeVideoSourceMac::createForMockDisplayCapturer(String { device.persistentId() }, AtomString { device.label() }, String { }, pageIdentifier))
+MockDisplayCapturer::MockDisplayCapturer(CapturerObserver& observer, const CaptureDevice& device, std::optional<PageIdentifier> pageIdentifier)
+    : DisplayCaptureSourceCocoa::Capturer(observer)
+    , m_source(MockRealtimeVideoSourceMac::createForMockDisplayCapturer(String { device.persistentId() }, AtomString { device.label() }, MediaDeviceHashSalts { "persistent"_s, "ephemeral"_s }, pageIdentifier))
+    , m_readyTimer(*this, &MockDisplayCapturer::readyTimerFired)
 {
 }
 
 bool MockDisplayCapturer::start()
 {
     ASSERT(m_settings.frameRate());
+
+    ASSERT(!m_whenReadyCallback);
+    ASSERT(!m_isRunning);
+
+    m_isRunning = true;
     m_source->start();
     return true;
 }
 
+void MockDisplayCapturer::stop()
+{
+    ASSERT(!m_whenReadyCallback);
+
+    m_isRunning = false;
+    m_source->stop();
+}
+
+void MockDisplayCapturer::whenReady(CompletionHandler<void(CaptureSourceError&&)>&& callback)
+{
+    ASSERT(!m_isRunning);
+
+    m_whenReadyCallback = WTFMove(callback);
+    m_readyTimer.startOneShot(50_ms);
+}
+
+void MockDisplayCapturer::readyTimerFired()
+{
+    ASSERT(!m_isRunning);
+
+    configurationChanged();
+    m_whenReadyCallback({ });
+}
+
 void MockDisplayCapturer::commitConfiguration(const RealtimeMediaSourceSettings& settings)
 {
     // FIXME: Update m_source width, height and frameRate according settings
@@ -173,27 +234,50 @@ IntSize MockDisplayCapturer::intrinsicSize() const
     auto& properties = std::get<MockDisplayProperties>(device->properties);
     return properties.defaultSize;
 }
-#endif // PLATFORM(MAC)
+
+void MockDisplayCapturer::triggerMockCaptureConfigurationChange()
+{
+    auto deviceId = m_source->persistentID();
+    auto device = MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(deviceId.startsWith("WINDOW"_s) ? "WINDOW-2"_s : "SCREEN-2"_s);
+    ASSERT(device);
+    if (!device)
+        return;
+
+    bool isStarted = m_source->isProducingData();
+    auto pageIdentifier = m_source->pageIdentifier();
+    m_source = MockRealtimeVideoSourceMac::createForMockDisplayCapturer(String { device->persistentId }, AtomString { device->label }, MediaDeviceHashSalts { "persistent"_s, "ephemeral"_s }, pageIdentifier);
+    if (isStarted)
+        m_source->start();
+
+    configurationChanged();
+}
+#endif // PLATFORM(COCOA)
 
 class MockRealtimeDisplaySourceFactory : public DisplayCaptureFactory {
 public:
-    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
+    static MockRealtimeDisplaySourceFactory& singleton();
+
+    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice& device, MediaDeviceHashSalts&& hashSalts, const MediaConstraints* constraints, std::optional<PageIdentifier> pageIdentifier) final
     {
         if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(device.type(), device.persistentId()))
-            return { "Unable to find mock display device with given persistentID"_s };
+            return CaptureSourceOrError({ "Unable to find mock display device with given persistentID"_s, MediaAccessDenialReason::PermissionDenied });
 
         switch (device.type()) {
         case CaptureDevice::DeviceType::Screen:
-        case CaptureDevice::DeviceType::Window:
-#if PLATFORM(MAC)
-            return DisplayCaptureSourceCocoa::create(UniqueRef<DisplayCaptureSourceCocoa::Capturer>(makeUniqueRef<MockDisplayCapturer>(device, pageIdentifier)), device, WTFMove(hashSalt), constraints, pageIdentifier);
+        case CaptureDevice::DeviceType::Window: {
+#if PLATFORM(COCOA)
+            return DisplayCaptureSourceCocoa::create([this, &device, pageIdentifier] (auto& observer) {
+                auto capturer = makeUniqueRefWithoutRefCountedCheck<MockDisplayCapturer>(observer, device, pageIdentifier);
+                m_capturer = capturer.get();
+                return capturer;
+            }, device, WTFMove(hashSalts), constraints, pageIdentifier);
 #elif USE(GSTREAMER)
-            UNUSED_PARAM(pageIdentifier);
-            return MockDisplayCaptureSourceGStreamer::create(device, WTFMove(hashSalt), constraints);
+            return MockDisplayCaptureSourceGStreamer::create(device, WTFMove(hashSalts), constraints, pageIdentifier);
 #else
-            return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
+            return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalts), constraints, pageIdentifier);
 #endif
             break;
+        }
         case CaptureDevice::DeviceType::Microphone:
         case CaptureDevice::DeviceType::Speaker:
         case CaptureDevice::DeviceType::Camera:
@@ -206,19 +290,37 @@ public:
         return { };
     }
 
+#if PLATFORM(COCOA)
+    WeakPtr<MockDisplayCapturer> latestCapturer() { return m_capturer; }
+#endif
+
 private:
     DisplayCaptureManager& displayCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().displayCaptureDeviceManager(); }
+#if PLATFORM(COCOA)
+    WeakPtr<MockDisplayCapturer> m_capturer;
+#endif
 };
 
-class MockRealtimeAudioSourceFactory final : public AudioCaptureFactory {
+class MockRealtimeAudioSourceFactory final
+#if PLATFORM(COCOA)
+    : public CoreAudioCaptureSourceFactory
+#else
+    : public AudioCaptureFactory
+#endif
+{
 public:
-    CaptureSourceOrError createAudioCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
+    CaptureSourceOrError createAudioCaptureSource(const CaptureDevice& device, MediaDeviceHashSalts&& hashSalts, const MediaConstraints* constraints, std::optional<PageIdentifier> pageIdentifier) final
     {
-        ASSERT(device.type() == CaptureDevice::DeviceType::Microphone);
-        if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType::Microphone, device.persistentId()))
-            return { "Unable to find mock microphone device with given persistentID"_s };
+        ASSERT(device.type() == CaptureDevice::DeviceType::Microphone || device.type() == CaptureDevice::DeviceType::Speaker);
+        if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(device.type(), device.persistentId()))
+            return CaptureSourceOrError({ "Unable to find mock microphone or speaker device with given persistentID"_s, MediaAccessDenialReason::PermissionDenied });
 
-        return MockRealtimeAudioSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
+        auto mock = MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(device.persistentId());
+        ASSERT(mock);
+        if (mock->flags.contains(MockMediaDevice::Flag::Invalid))
+            return CaptureSourceOrError({ "Invalid mock microphone or speaker device"_s, MediaAccessDenialReason::PermissionDenied });
+
+        return MockRealtimeAudioSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalts), constraints, pageIdentifier);
     }
 private:
     CaptureDeviceManager& audioCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().audioCaptureDeviceManager(); }
@@ -272,8 +374,12 @@ void MockRealtimeMediaSourceCenter::setMockRealtimeMediaSourceCenterEnabled(bool
     RealtimeMediaSourceCenter& center = RealtimeMediaSourceCenter::singleton();
 
     if (mock.m_isEnabled) {
-        if (mock.m_isMockAudioCaptureEnabled)
+        if (mock.m_isMockAudioCaptureEnabled) {
+#if PLATFORM(COCOA)
+            MockAudioSharedUnit::enable();
+#endif
             center.setAudioCaptureFactory(mock.audioCaptureFactory());
+        }
         if (mock.m_isMockVideoCaptureEnabled)
             center.setVideoCaptureFactory(mock.videoCaptureFactory());
         if (mock.m_isMockDisplayCaptureEnabled)
@@ -281,8 +387,12 @@ void MockRealtimeMediaSourceCenter::setMockRealtimeMediaSourceCenterEnabled(bool
         return;
     }
 
-    if (mock.m_isMockAudioCaptureEnabled)
+    if (mock.m_isMockAudioCaptureEnabled) {
+#if PLATFORM(COCOA)
+        MockAudioSharedUnit::disable();
+#endif
         center.unsetAudioCaptureFactory(mock.audioCaptureFactory());
+    }
     if (mock.m_isMockVideoCaptureEnabled)
         center.unsetVideoCaptureFactory(mock.videoCaptureFactory());
     if (mock.m_isMockDisplayCaptureEnabled)
@@ -299,12 +409,16 @@ static CaptureDevice toCaptureDevice(const MockMediaDevice& device)
     auto captureDevice = device.captureDevice();
     captureDevice.setEnabled(true);
     captureDevice.setIsMockDevice(true);
+
     return captureDevice;
 }
 
-static void createMockDevice(const MockMediaDevice& device)
+static void createMockDevice(const MockMediaDevice& device, bool isDefault)
 {
-    deviceListForDevice(device).append(toCaptureDevice(device));
+    if (isDefault)
+        deviceListForDevice(device).insert(0, toCaptureDevice(device));
+    else
+        deviceListForDevice(device).append(toCaptureDevice(device));
 }
 
 void MockRealtimeMediaSourceCenter::resetDevices()
@@ -319,6 +433,29 @@ void MockRealtimeMediaSourceCenter::setMockCaptureDevicesInterrupted(bool isCame
     MockRealtimeAudioSource::setIsInterrupted(isMicrophoneInterrupted);
 }
 
+void MockRealtimeMediaSourceCenter::triggerMockCaptureConfigurationChange(bool forCamera, bool forMicrophone, bool forDisplay)
+{
+    if (forCamera)
+        MockRealtimeVideoSource::triggerCameraConfigurationChange();
+
+#if PLATFORM(COCOA)
+    if (forMicrophone) {
+        auto devices = audioCaptureDeviceManager().captureDevices();
+        if (devices.size() > 1) {
+            MockAudioSharedUnit::increaseBufferSize();
+            CoreAudioSharedUnit::singleton().handleNewCurrentMicrophoneDevice(WTFMove(devices[1]));
+        }
+    }
+    if (forDisplay) {
+        if (auto capturer = MockRealtimeDisplaySourceFactory::singleton().latestCapturer())
+            capturer->triggerMockCaptureConfigurationChange();
+    }
+#else
+    UNUSED_PARAM(forMicrophone);
+    UNUSED_PARAM(forDisplay);
+#endif
+}
+
 void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMockDevices)
 {
     microphoneDevices().clear();
@@ -327,6 +464,13 @@ void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMock
     displayDevices().clear();
 
     auto& mockDevices = devices();
+    for (auto& device : mockDevices) {
+        auto& persistentId = device.persistentId;
+        if (!newMockDevices.containsIf([&persistentId](auto& newDevice) -> bool {
+            return newDevice.persistentId == persistentId;
+        }))
+            RealtimeMediaSourceCenter::singleton().captureDeviceWillBeRemoved(persistentId);
+    }
     mockDevices = WTFMove(newMockDevices);
 
     auto& map = deviceMap();
@@ -334,16 +478,26 @@ void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMock
 
     for (const auto& device : mockDevices) {
         map.add(device.persistentId, device);
-        createMockDevice(device);
+        createMockDevice(device, false);
     }
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
+static bool shouldBeDefaultDevice(const MockMediaDevice& device)
+{
+    auto* cameraProperties = device.cameraProperties();
+    return cameraProperties && cameraProperties->facingMode == VideoFacingMode::Unknown;
+}
+
 void MockRealtimeMediaSourceCenter::addDevice(const MockMediaDevice& device)
 {
-    devices().append(device);
+    bool isDefault = shouldBeDefaultDevice(device);
+    if (isDefault)
+        devices().insert(0, device);
+    else
+        devices().append(device);
     deviceMap().set(device.persistentId, device);
-    createMockDevice(device);
+    createMockDevice(device, isDefault);
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
@@ -354,6 +508,7 @@ void MockRealtimeMediaSourceCenter::removeDevice(const String& persistentId)
     if (iterator == map.end())
         return;
 
+    RealtimeMediaSourceCenter::singleton().captureDeviceWillBeRemoved(persistentId);
     devices().removeFirstMatching([&persistentId](const auto& device) {
         return device.persistentId == persistentId;
     });
@@ -366,6 +521,25 @@ void MockRealtimeMediaSourceCenter::removeDevice(const String& persistentId)
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
+void MockRealtimeMediaSourceCenter::setDeviceIsEphemeral(const String& persistentId, bool isEphemeral)
+{
+    ASSERT(!persistentId.isEmpty());
+
+    auto& map = deviceMap();
+    auto iterator = map.find(persistentId);
+    if (iterator == map.end())
+        return;
+
+    MockMediaDevice device = iterator->value;
+    if (isEphemeral)
+        device.flags.add(MockMediaDevice::Flag::Ephemeral);
+    else
+        device.flags.remove(MockMediaDevice::Flag::Ephemeral);
+
+    removeDevice(persistentId);
+    addDevice(device);
+}
+
 std::optional<MockMediaDevice> MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(const String& id)
 {
     ASSERT(!id.isEmpty());
@@ -435,7 +609,7 @@ Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::displayDevices()
         Vector<CaptureDevice> displayDevices;
         for (const auto& device : devices()) {
             if (device.isDisplay())
-                displayDevices.append(device.captureDevice());
+                displayDevices.append(toCaptureDevice(device));
         }
         return displayDevices;
     }();
@@ -456,19 +630,13 @@ VideoCaptureFactory& MockRealtimeMediaSourceCenter::videoCaptureFactory()
 
 DisplayCaptureFactory& MockRealtimeMediaSourceCenter::displayCaptureFactory()
 {
-    static NeverDestroyed<MockRealtimeDisplaySourceFactory> factory;
-    return factory.get();
+    return MockRealtimeDisplaySourceFactory::singleton();
 }
 
-void MockRealtimeMediaSourceCenter::MockDisplayCaptureDeviceManager::windowDevices(Vector<DisplayCaptureManager::WindowCaptureDevice>& windowDevices)
+MockRealtimeDisplaySourceFactory& MockRealtimeDisplaySourceFactory::singleton()
 {
-    auto devices = MockRealtimeMediaSourceCenter::displayDevices();
-    for (auto device : devices) {
-        if (!device.enabled() || device.type() != CaptureDevice::DeviceType::Window)
-            continue;
-
-        windowDevices.append({ WTFMove(device), "Mock Application"_s });
-    }
+    static NeverDestroyed<MockRealtimeDisplaySourceFactory> factory;
+    return factory.get();
 }
 
 } // namespace WebCore
